@@ -1,6 +1,6 @@
 /*
  * playlist-new.c
- * Copyright 2009 John Lindgren
+ * Copyright 2009-2010 John Lindgren
  *
  * This file is part of Audacious.
  *
@@ -19,20 +19,30 @@
  * using our public API to be a derived work.
  */
 
-#include <glib.h>
+#include <assert.h>
 #include <inttypes.h>
 #include <stdlib.h>
 #include <time.h>
 
+#include <glib.h>
+
+#include <libaudcore/audstrings.h>
+#include <libaudcore/hook.h>
+#include <libaudcore/tuple_formatter.h>
+
+#include "audconfig.h"
 #include "config.h"
-#include "audstrings.h"
 #include "i18n.h"
+#include "main.h"
+#include "misc.h"
 #include "playback.h"
-#include "playlist-new.h"
+#include "playlist.h"
 #include "playlist-utils.h"
 #include "plugin.h"
-#include "probe.h"
 
+#define SCAN_DEBUG(...)
+
+#define SCAN_THREADS 4
 #define STATE_FILE "playlist-state"
 
 #define DECLARE_PLAYLIST \
@@ -107,7 +117,7 @@ struct entry
     gint length;
     gboolean failed;
     gboolean selected;
-    gboolean shuffle_num;
+    gint shuffle_num;
     gboolean queued;
     gboolean segmented;
     gint start;
@@ -122,7 +132,7 @@ struct playlist
     struct index *entries;
     struct entry *position;
     gint selected_count;
-    gint shuffle_num, last_shuffle_num;
+    gint last_shuffle_num;
     GList *queued;
     gint64 total_length;
     gint64 selected_length;
@@ -135,15 +145,14 @@ static struct playlist *playing_playlist;
 static gint update_source, update_level;
 static gint scan_source;
 static GMutex * scan_mutex;
-static GCond * scan_cond;
-static const gchar * scan_filename;
-static InputPlugin * scan_decoder;
-static Tuple * scan_tuple;
-static GThread * scan_thread;
-static gint scan_position, updated_ago;
-
-static gint(*current_filename_compare) (const gchar * a, const gchar * b);
-static gint(*current_tuple_compare) (const Tuple * a, const Tuple * b);
+static GCond * scan_conds[SCAN_THREADS];
+static const gchar * scan_filenames[SCAN_THREADS];
+static InputPlugin * scan_decoders[SCAN_THREADS];
+static Tuple * scan_tuples[SCAN_THREADS];
+static gboolean scan_quit;
+static GThread * scan_threads[SCAN_THREADS];
+static gint scan_positions[SCAN_THREADS];
+gint updated_ago;
 
 static void * scanner (void * unused);
 
@@ -159,18 +168,12 @@ static gchar *title_from_tuple(Tuple * tuple)
 
 static void entry_set_tuple_real (struct entry * entry, Tuple * tuple)
 {
-    gint start;
-    gint end;
-
-    if (entry->segmented == TRUE && entry->tuple)
+    /* Hack: We cannot refresh segmented entries (since their info is read from
+     * the cue sheet when it is first loaded), so leave them alone. -jlindgren */
+    if (entry->segmented)
     {
-        if (entry->end > 0)
-            return;
-
-        entry->length = tuple_get_int(tuple, FIELD_LENGTH, NULL);
-        entry->length -= entry->start;
-
-        tuple_associate_int(entry->tuple, FIELD_LENGTH, NULL, entry->length);
+        if (tuple != NULL)
+            tuple_free (tuple);
 
         return;
     }
@@ -194,18 +197,19 @@ static void entry_set_tuple_real (struct entry * entry, Tuple * tuple)
         entry->length = tuple_get_int (tuple, FIELD_LENGTH, NULL);
         entry->length = MAX (entry->length, 0);
 
-        if ((tuple_get_int (tuple, FIELD_SEGMENT_START, NULL)) || (tuple_get_int (tuple, FIELD_SEGMENT_END, NULL)))
+        if (tuple_get_value_type (tuple, FIELD_SEGMENT_START, NULL) == TUPLE_INT)
         {
-            start = tuple_get_int (tuple, FIELD_SEGMENT_START, NULL);
-            end = tuple_get_int (tuple, FIELD_SEGMENT_END, NULL);
-
-            if (start == end)
-                end = 0;
-
             entry->segmented = TRUE;
-            entry->start = start ? start : 0;
-            entry->end = end ? end : -1;
+            entry->start = tuple_get_int (tuple, FIELD_SEGMENT_START, NULL);
+
+            if (tuple_get_value_type (tuple, FIELD_SEGMENT_END, NULL) ==
+             TUPLE_INT)
+                entry->end = tuple_get_int (tuple, FIELD_SEGMENT_END, NULL);
+            else
+                entry->end = -1;
         }
+        else
+            entry->segmented = FALSE;
     }
 }
 
@@ -231,6 +235,12 @@ static void entry_set_tuple (struct playlist * playlist, struct entry * entry,
     }
 }
 
+static void entry_set_failed (struct playlist * playlist, struct entry * entry)
+{
+    entry_set_tuple (playlist, entry, tuple_new_from_filename (entry->filename));
+    entry->failed = TRUE;
+}
+
 static struct entry *entry_new(gchar * filename, InputPlugin * decoder, Tuple * tuple)
 {
     struct entry *entry = g_malloc(sizeof(struct entry));
@@ -262,15 +272,15 @@ static void entry_free(struct entry *entry)
     g_free(entry);
 }
 
-static void entry_check_has_decoder (struct entry * entry)
+static void entry_check_has_decoder (struct playlist * playlist, struct entry *
+ entry)
 {
     if (entry->decoder != NULL || entry->failed)
         return;
 
-    entry->decoder = file_probe (entry->filename, TRUE);
-
-    if (entry->decoder == NULL)
-        entry->failed = TRUE;
+    entry->decoder = file_find_decoder (entry->filename, FALSE);
+    if (! entry->decoder)
+        entry_set_failed (playlist, entry);
 }
 
 static struct playlist *playlist_new(void)
@@ -283,7 +293,6 @@ static struct playlist *playlist_new(void)
     playlist->entries = index_new();
     playlist->position = NULL;
     playlist->selected_count = 0;
-    playlist->shuffle_num = 0;
     playlist->last_shuffle_num = 0;
     playlist->queued = NULL;
     playlist->total_length = 0;
@@ -365,101 +374,111 @@ static void queue_update (gint level)
          NULL);
 }
 
-static Tuple * get_tuple (const gchar * filename, InputPlugin * decoder)
-{
-    if (decoder->get_song_tuple != NULL)
-        return decoder->get_song_tuple (filename);
-
-    if (decoder->probe_for_tuple != NULL)
-    {
-        VFSFile * handle = vfs_fopen (filename, "r");
-        Tuple * tuple;
-
-        if (handle == NULL)
-            return NULL;
-
-        tuple = decoder->probe_for_tuple (filename, handle);
-        vfs_fclose (handle);
-        return tuple;
-    }
-
-    return NULL;
-}
-
 /* scan_mutex must be locked! */
 void scan_receive (void)
 {
-    struct entry * entry = index_get (active_playlist->entries, scan_position);
+    for (gint i = 0; i < SCAN_THREADS; i ++)
+    {
+        struct entry * entry;
 
-    entry_set_tuple (active_playlist, entry, scan_tuple);
+        if (! scan_filenames[i] || scan_decoders[i])
+            continue; /* thread not in use or still working */
 
-    if (scan_tuple == NULL)
-        entry->failed = TRUE;
+        SCAN_DEBUG ("receive (#%d): %d\n", i, scan_positions[i]);
+        entry = index_get (active_playlist->entries, scan_positions[i]);
 
-    scan_filename = NULL;
-    scan_decoder = NULL;
-    scan_tuple = NULL;
+        if (scan_tuples[i])
+            entry_set_tuple (active_playlist, entry, scan_tuples[i]);
+        else
+            entry_set_failed (active_playlist, entry);
+
+        scan_filenames[i] = NULL;
+        scan_tuples[i] = NULL;
+
+        updated_ago ++;
+    }
 }
 
 static gboolean scan_next (void * unused)
 {
-    gint entries;
+    gint entries = index_count (active_playlist->entries);
+    gint search = 0;
 
     g_mutex_lock (scan_mutex);
 
-    if (scan_filename != NULL)
+    if (scan_source)
     {
-        scan_receive ();
-        scan_position ++;
-        updated_ago ++;
+        g_source_remove (scan_source);
+        scan_source = 0;
     }
 
-    entries = index_count (active_playlist->entries);
+    scan_receive ();
 
-    for (; scan_position < entries; scan_position ++)
+    for (gint i = 0; i < SCAN_THREADS; i ++)
+        search = MAX (search, scan_positions[i] + 1);
+
+    for (gint i = 0; i < SCAN_THREADS; i ++)
     {
-        struct entry * entry = index_get (active_playlist->entries,
-         scan_position);
+        if (scan_filenames[i])
+            continue; /* thread already in use */
 
-        if (entry->tuple != NULL)
-            continue;
-
-        entry_check_has_decoder (entry);
-
-        if (! entry->failed)
+        for (; search < entries; search ++)
         {
-            scan_filename = entry->filename;
-            scan_decoder = entry->decoder;
-            g_cond_signal (scan_cond);
+            struct entry * entry = index_get (active_playlist->entries, search);
+
+            if (entry->tuple)
+                continue;
+
+            entry_check_has_decoder (active_playlist, entry);
+            if (entry->failed)
+                continue;
+
+            SCAN_DEBUG ("start (#%d): %d\n", i, search);
+            scan_positions[i] = search;
+            scan_filenames[i] = entry->filename;
+            scan_decoders[i] = entry->decoder;
+            g_cond_signal (scan_conds[i]);
+
+            search ++;
             break;
         }
     }
 
-    if (updated_ago >= 10 || (scan_position == entries && updated_ago > 0))
+    if (updated_ago >= 10 || (search == entries && updated_ago > 0))
     {
+        SCAN_DEBUG ("queue update\n");
         queue_update (PLAYLIST_UPDATE_METADATA);
         updated_ago = 0;
     }
 
-    scan_source = 0;
     g_mutex_unlock (scan_mutex);
     return FALSE;
 }
 
 static void scan_continue (void)
 {
-    scan_source = g_idle_add_full (G_PRIORITY_LOW, scan_next, NULL, NULL);
+    SCAN_DEBUG ("scan_continue\n");
+    if (! scan_source)
+        scan_source = g_idle_add_full (G_PRIORITY_LOW, scan_next, NULL, NULL);
 }
 
 static void scan_reset (void)
 {
-    scan_position = 0;
+    SCAN_DEBUG ("scan_reset\n");
+
+    for (gint i = 0; i < SCAN_THREADS; i ++)
+    {
+        assert (! scan_filenames[i]); /* scan in progress == very, very bad */
+        scan_positions[i] = -1;
+    }
+
     updated_ago = 0;
     scan_continue ();
 }
 
 static void scan_stop (void)
 {
+    SCAN_DEBUG ("scan_stop\n");
     g_mutex_lock (scan_mutex);
 
     if (scan_source != 0)
@@ -468,46 +487,132 @@ static void scan_stop (void)
         scan_source = 0;
     }
 
-    if (scan_filename != NULL)
-        scan_receive ();
+    for (gint i = 0; i < SCAN_THREADS; i ++)
+    {
+        if (! scan_filenames[i])
+            continue;
 
+        while (scan_decoders[i])
+        {
+            SCAN_DEBUG ("wait for stop (#%d)\n", i);
+            g_cond_wait (scan_conds[i], scan_mutex);
+        }
+    }
+
+    scan_receive ();
     g_mutex_unlock (scan_mutex);
 }
 
-/* scan_mutex must be locked! */
-static void * scanner (void * unused)
+static void * scanner (void * data)
 {
-    for (;;)
+    gint i = GPOINTER_TO_INT (data);
+
+    g_mutex_lock (scan_mutex);
+    g_cond_signal (scan_conds[i]);
+
+    while (1)
     {
-        g_cond_wait (scan_cond, scan_mutex);
+        SCAN_DEBUG ("scanner (#%d): wait\n", i);
+        g_cond_wait (scan_conds[i], scan_mutex);
 
-        if (scan_filename == NULL)
-            return NULL;
+        if (scan_quit)
+            break;
 
-        scan_tuple = get_tuple (scan_filename, scan_decoder);
+        if (! scan_filenames[i])
+        {
+            SCAN_DEBUG ("scanner (#%d): idle\n", i);
+            continue;
+        }
+
+        SCAN_DEBUG ("scanner (#%d): scan %s\n", i, scan_filenames[i]);
+        scan_tuples[i] = file_read_tuple (scan_filenames[i], scan_decoders[i]);
+        scan_decoders[i] = NULL;
+        g_cond_signal (scan_conds[i]);
         scan_continue ();
     }
+
+    SCAN_DEBUG ("scanner (#%d): exit\n", i);
+    g_mutex_unlock (scan_mutex);
+    return NULL;
+}
+
+/* As soon as we know the caller is looking for metadata, we start the threaded
+ * scanner.  Though it may be faster in the short run simply to scan the entry
+ * we are concerned with in the main thread, this is better in the long run
+ * because the scanner can work on the following entries while the caller is
+ * processing this one. */
+static gboolean scan_threaded (struct playlist * playlist, struct entry * entry)
+{
+    gint i;
+
+    if (playlist != active_playlist)
+        return FALSE;
+
+    scan_next (NULL);
+
+    if (entry->tuple)
+        return TRUE;
+
+    for (i = 0; i < SCAN_THREADS; i ++)
+    {
+        if (entry->number == scan_positions[i])
+            goto FOUND;
+    }
+
+    SCAN_DEBUG ("manual scan of %d\n", entry->number);
+    return FALSE;
+
+FOUND:
+    SCAN_DEBUG ("threaded scan (#%d) of %d\n", i, entry->number);
+    g_mutex_lock (scan_mutex);
+    scan_receive ();
+
+    while (scan_filenames[i])
+    {
+        SCAN_DEBUG ("wait (#%d) for %d\n", i, entry->number);
+        g_cond_wait (scan_conds[i], scan_mutex);
+        scan_receive ();
+    }
+
+    g_mutex_unlock (scan_mutex);
+    return TRUE;
 }
 
 static void check_scanned (struct playlist * playlist, struct entry * entry)
 {
-    if (entry->tuple != NULL || entry->failed)
+    if (entry->tuple)
+        return;
+    if (scan_threaded (playlist, entry))
         return;
 
-    METADATA_WILL_CHANGE;
+    entry_check_has_decoder (playlist, entry);
+    if (entry->failed)
+        return;
 
-    entry_check_has_decoder (entry);
+    entry_set_tuple (playlist, entry, file_read_tuple (entry->filename,
+     entry->decoder));
+    if (! entry->tuple)
+        entry_set_failed (playlist, entry);
 
-    if (entry->tuple == NULL && ! entry->failed) /* scanner did it for us? */
+    queue_update (PLAYLIST_UPDATE_METADATA);
+}
+
+static void check_selected_scanned (struct playlist * playlist)
+{
+    gint entries = index_count (playlist->entries);
+    for (gint count = 0; count < entries; count++)
     {
-        entry_set_tuple (playlist, entry, get_tuple (entry->filename,
-         entry->decoder));
-
-        if (entry->tuple == NULL)
-            entry->failed = TRUE;
+        struct entry * entry = index_get (playlist->entries, count);
+        if (entry->selected)
+            check_scanned (playlist, entry);
     }
+}
 
-    METADATA_HAS_CHANGED;
+static void check_all_scanned (struct playlist * playlist)
+{
+    gint entries = index_count (playlist->entries);
+    for (gint count = 0; count < entries; count++)
+        check_scanned (playlist, index_get (playlist->entries, count));
 }
 
 void playlist_init (void)
@@ -527,13 +632,24 @@ void playlist_init (void)
     update_level = 0;
 
     scan_mutex = g_mutex_new ();
-    scan_cond = g_cond_new ();
-    scan_filename = NULL;
-    scan_decoder = NULL;
-    scan_tuple = NULL;
+    memset (scan_filenames, 0, sizeof scan_filenames);
+    memset (scan_decoders, 0, sizeof scan_decoders);
+    memset (scan_tuples, 0, sizeof scan_tuples);
     scan_source = 0;
+    scan_quit = FALSE;
+
     g_mutex_lock (scan_mutex);
-    scan_thread = g_thread_create (scanner, NULL, TRUE, NULL);
+
+    for (gint i = 0; i < SCAN_THREADS; i ++)
+    {
+        scan_conds[i] = g_cond_new ();
+        scan_threads[i] = g_thread_create (scanner, GINT_TO_POINTER (i), TRUE,
+         NULL);
+        g_cond_wait (scan_conds[i], scan_mutex);
+    }
+
+    g_mutex_unlock (scan_mutex);
+
     scan_reset ();
 }
 
@@ -542,11 +658,18 @@ void playlist_end(void)
     gint count;
 
     scan_stop ();
-    g_mutex_lock (scan_mutex);
-    scan_filename = NULL; /* tell scanner to quit */
-    g_cond_signal (scan_cond);
-    g_mutex_unlock (scan_mutex);
-    g_thread_join (scan_thread);
+    scan_quit = TRUE;
+
+    for (gint i = 0; i < SCAN_THREADS; i ++)
+    {
+        g_mutex_lock (scan_mutex);
+        g_cond_signal (scan_conds[i]);
+        g_mutex_unlock (scan_mutex);
+        g_thread_join (scan_threads[i]);
+        g_cond_free (scan_conds[i]);
+    }
+
+    g_mutex_free (scan_mutex);
 
     if (update_source != 0)
         g_source_remove(update_source);
@@ -580,6 +703,41 @@ void playlist_insert(gint at)
     hook_call ("playlist insert", GINT_TO_POINTER (at));
 }
 
+void playlist_reorder (gint from, gint to, gint count)
+{
+    struct index * displaced;
+
+    g_return_if_fail (from >= 0 && from + count <= index_count (playlists));
+    g_return_if_fail (to >= 0 && to + count <= index_count (playlists));
+    g_return_if_fail (count >= 0);
+
+    PLAYLIST_WILL_CHANGE;
+
+    displaced = index_new ();
+
+    if (to < from)
+        index_copy_append (playlists, to, displaced, from - to);
+    else
+        index_copy_append (playlists, from + count, displaced, to - from);
+
+    index_move (playlists, from, to, count);
+
+    if (to < from)
+    {
+        index_copy_set (displaced, 0, playlists, to + count, from - to);
+        number_playlists (to, from + count - to);
+    }
+    else
+    {
+        index_copy_set (displaced, 0, playlists, from, to - from);
+        number_playlists (from, to + count - from);
+    }
+
+    index_free (displaced);
+
+    PLAYLIST_HAS_CHANGED;
+}
+
 void playlist_delete (gint playlist_num)
 {
     DECLARE_PLAYLIST;
@@ -617,12 +775,12 @@ void playlist_set_filename(gint playlist_num, const gchar * filename)
     DECLARE_PLAYLIST;
 
     LOOKUP_PLAYLIST;
-    METADATA_WILL_CHANGE;
+    PLAYLIST_WILL_CHANGE;
 
     g_free(playlist->filename);
     playlist->filename = g_strdup(filename);
 
-    METADATA_HAS_CHANGED;
+    PLAYLIST_HAS_CHANGED;
 }
 
 const gchar *playlist_get_filename(gint playlist_num)
@@ -639,12 +797,12 @@ void playlist_set_title(gint playlist_num, const gchar * title)
     DECLARE_PLAYLIST;
 
     LOOKUP_PLAYLIST;
-    METADATA_WILL_CHANGE;
+    PLAYLIST_WILL_CHANGE;
 
     g_free(playlist->title);
     playlist->title = g_strdup(title);
 
-    METADATA_HAS_CHANGED;
+    PLAYLIST_HAS_CHANGED;
 }
 
 const gchar *playlist_get_title(gint playlist_num)
@@ -677,7 +835,10 @@ void playlist_set_playing(gint playlist_num)
 {
     DECLARE_PLAYLIST;
 
-    LOOKUP_PLAYLIST;
+    if (playlist_num == -1)
+        playlist = NULL;
+    else
+        LOOKUP_PLAYLIST;
 
     if (playing_playlist != NULL && playback_get_playing ())
         playback_stop();
@@ -690,8 +851,13 @@ gint playlist_get_playing(void)
     return (playing_playlist == NULL) ? -1 : playing_playlist->number;
 }
 
+/* If we are already at the song or it is already at the top of the shuffle
+ * list, we let it be.  Otherwise, we move it to the top. */
 static void set_position (struct playlist * playlist, struct entry * entry)
 {
+    if (entry == playlist->position)
+        return;
+
     playlist->position = entry;
 
     if (entry == NULL)
@@ -702,8 +868,6 @@ static void set_position (struct playlist * playlist, struct entry * entry)
         playlist->last_shuffle_num ++;
         entry->shuffle_num = playlist->last_shuffle_num;
     }
-
-    playlist->shuffle_num = entry->shuffle_num;
 }
 
 gint playlist_entry_count(gint playlist_num)
@@ -718,12 +882,14 @@ gint playlist_entry_count(gint playlist_num)
 static void make_entries (gchar * filename, InputPlugin * decoder, Tuple *
  tuple, struct index * list)
 {
-    if (tuple == NULL && decoder == NULL)
-        decoder = file_probe (filename, TRUE);
+    uri_check_utf8 (& filename, TRUE);
 
-    if (tuple == NULL && decoder != NULL && decoder->have_subtune &&
-     decoder->get_song_tuple != NULL)
-        tuple = decoder->get_song_tuple (filename);
+    if (tuple == NULL && decoder == NULL)
+        decoder = file_find_decoder (filename, TRUE);
+
+    if (tuple == NULL && decoder != NULL && decoder->have_subtune && strchr
+     (filename, '?') == NULL)
+        tuple = file_read_tuple (filename, decoder);
 
     if (tuple != NULL && tuple->nsubtunes > 0)
     {
@@ -863,7 +1029,7 @@ InputPlugin *playlist_entry_get_decoder(gint playlist_num, gint entry_num)
 
     LOOKUP_PLAYLIST_ENTRY_RET (NULL);
 
-    entry_check_has_decoder (entry);
+    entry_check_has_decoder (playlist, entry);
 
     return entry->decoder;
 }
@@ -880,57 +1046,41 @@ void playlist_entry_set_tuple (gint playlist_num, gint entry_num, Tuple * tuple)
     METADATA_HAS_CHANGED;
 }
 
-const Tuple *playlist_entry_get_tuple(gint playlist_num, gint entry_num)
+const Tuple * playlist_entry_get_tuple (gint playlist_num, gint entry_num,
+ gboolean fast)
 {
     DECLARE_PLAYLIST_ENTRY;
-
     LOOKUP_PLAYLIST_ENTRY_RET (NULL);
 
-    check_scanned (playlist, entry);
+    if (! fast)
+        check_scanned (playlist, entry);
+
     return entry->tuple;
 }
 
-const gchar *playlist_entry_get_title(gint playlist_num, gint entry_num)
+const gchar * playlist_entry_get_title (gint playlist_num, gint entry_num,
+ gboolean fast)
 {
     DECLARE_PLAYLIST_ENTRY;
-
     LOOKUP_PLAYLIST_ENTRY_RET (NULL);
 
-    check_scanned (playlist, entry);
+    if (! fast)
+        check_scanned (playlist, entry);
+
     return (entry->title == NULL) ? entry->filename : entry->title;
 }
 
-gint playlist_entry_get_length(gint playlist_num, gint entry_num)
+gint playlist_entry_get_length (gint playlist_num, gint entry_num, gboolean fast)
 {
     DECLARE_PLAYLIST_ENTRY;
-
     LOOKUP_PLAYLIST_ENTRY_RET (0);
 
-    check_scanned (playlist, entry);
+    if (! fast)
+        check_scanned (playlist, entry);
+
     return entry->length;
 }
 
-void playlist_entry_set_segmentation(gint playlist_num, gint entry_num, gint start, gint end)
-{
-    DECLARE_PLAYLIST_ENTRY;
-
-    LOOKUP_PLAYLIST_ENTRY;
-    if (entry == NULL)
-        return;
-
-    if (start != -1)
-    {
-        entry->segmented = TRUE;
-        entry->start = start;
-        entry->end = end;
-    }
-    else
-    {
-        entry->segmented = FALSE;
-        entry->start = entry->end = -1;
-    }
-}
-
 gboolean playlist_entry_is_segmented(gint playlist_num, gint entry_num)
 {
     DECLARE_PLAYLIST_ENTRY;
@@ -1136,64 +1286,6 @@ gint playlist_shift (gint playlist_num, gint entry_num, gint distance)
     return shift;
 }
 
-gint playlist_shift_selected(gint playlist_num, gint distance)
-{
-    DECLARE_PLAYLIST;
-    gint entries, shifted, count;
-    struct entry *entry;
-
-    LOOKUP_PLAYLIST_RET (0);
-    PLAYLIST_WILL_CHANGE;
-
-    entries = index_count(playlist->entries);
-
-    if (entries == 0)
-        return 0;
-
-    for (shifted = 0; shifted > distance; shifted--)
-    {
-        entry = index_get(playlist->entries, 0);
-
-        if (entry->selected)
-            break;
-
-        for (count = 1; count < entries; count++)
-        {
-            entry = index_get(playlist->entries, count);
-
-            if (!entry->selected)
-                continue;
-
-            index_set(playlist->entries, count, index_get(playlist->entries, count - 1));
-            index_set(playlist->entries, count - 1, entry);
-        }
-    }
-
-    for (; shifted < distance; shifted++)
-    {
-        entry = index_get(playlist->entries, entries - 1);
-
-        if (entry->selected)
-            break;
-
-        for (count = entries - 1; count--;)
-        {
-            entry = index_get(playlist->entries, count);
-
-            if (!entry->selected)
-                continue;
-
-            index_set(playlist->entries, count, index_get(playlist->entries, count + 1));
-            index_set(playlist->entries, count + 1, entry);
-        }
-    }
-
-    number_entries(playlist, 0, entries);
-
-    PLAYLIST_HAS_CHANGED;
-    return shifted;
-}
-
 void playlist_delete_selected(gint playlist_num)
 {
     DECLARE_PLAYLIST;
@@ -1234,6 +1326,7 @@ void playlist_delete_selected(gint playlist_num)
     playlist->entries = others;
 
     number_entries(playlist, 0, index_count(playlist->entries));
+    playlist->selected_count = 0;
     playlist->selected_length = 0;
 
     if (stop && playback_get_playing ())
@@ -1266,60 +1359,71 @@ void playlist_reverse(gint playlist_num)
     PLAYLIST_HAS_CHANGED;
 }
 
-void playlist_randomize(gint playlist_num)
+void playlist_randomize (gint playlist_num)
 {
     DECLARE_PLAYLIST;
-    gint entries, count;
-
     LOOKUP_PLAYLIST;
     PLAYLIST_WILL_CHANGE;
 
-    entries = index_count(playlist->entries);
+    gint entries = index_count (playlist->entries);
 
-    for (count = 0; count < entries; count++)
+    for (gint i = 0; i < entries; i ++)
     {
-        gint replace = count + random() % (entries - count);
-        struct entry *entry = index_get(playlist->entries, replace);
+        gint j = i + random () % (entries - i);
 
-        index_set(playlist->entries, replace, index_get(playlist->entries, count));
-        index_set(playlist->entries, count, entry);
+        struct entry * entry = index_get (playlist->entries, j);
+        index_set (playlist->entries, j, index_get (playlist->entries, i));
+        index_set (playlist->entries, i, entry);
     }
 
-    number_entries(playlist, 0, entries);
-
+    number_entries (playlist, 0, entries);
     PLAYLIST_HAS_CHANGED;
 }
 
-static gint filename_compare(const void **a, const void **b)
+static gint filename_compare (const void * _a, const void * _b, void * _compare)
 {
-    const struct entry *entry_a = *a, *entry_b = *b;
+    const struct entry * a = _a, * b = _b;
+    gint (* compare) (const gchar * a, const gchar * b) = _compare;
 
-    return current_filename_compare(entry_a->filename, entry_b->filename);
+    gint diff = compare (a->filename, b->filename);
+    if (diff)
+        return diff;
+
+    /* preserve order of "equal" entries */
+    return a->number - b->number;
 }
 
-static gint tuple_compare(const void **a, const void **b)
+static gint tuple_compare (const void * _a, const void * _b, void * _compare)
 {
-    const struct entry *entry_a = *a, *entry_b = *b;
+    const struct entry * a = _a, * b = _b;
+    gint (* compare) (const Tuple * a, const Tuple * b) = _compare;
 
-    if (entry_a->tuple == NULL)
-        return (entry_b->tuple == NULL) ? 0 : -1;
-    if (entry_b->tuple == NULL)
+    if (a->tuple == NULL)
+        return (b->tuple == NULL) ? 0 : -1;
+    if (b->tuple == NULL)
         return 1;
 
-    return current_tuple_compare(entry_a->tuple, entry_b->tuple);
+    gint diff = compare (a->tuple, b->tuple);
+    if (diff)
+        return diff;
+
+    /* preserve order of "equal" entries */
+    return a->number - b->number;
 }
 
-static void sort(struct playlist *playlist, gint(*compare) (const void **a, const void **b))
+static void sort (struct playlist * playlist, gint (* compare) (const void * a,
+ const void * b, void * inner), void * inner)
 {
     PLAYLIST_WILL_CHANGE;
 
-    index_sort(playlist->entries, compare);
-    number_entries(playlist, 0, index_count(playlist->entries));
+    index_sort_with_data (playlist->entries, compare, inner);
+    number_entries (playlist, 0, index_count (playlist->entries));
 
     PLAYLIST_HAS_CHANGED;
 }
 
-static void sort_selected(struct playlist *playlist, gint(*compare) (const void **a, const void **b))
+static void sort_selected (struct playlist * playlist, gint (* compare)
+ (const void * a, const void * b, void * inner), void * inner)
 {
     gint entries, count, count2;
     struct index *selected;
@@ -1337,7 +1441,7 @@ static void sort_selected(struct playlist *playlist, gint(*compare) (const void
             index_append(selected, entry);
     }
 
-    index_sort(selected, compare);
+    index_sort_with_data (selected, compare, inner);
 
     count2 = 0;
 
@@ -1355,66 +1459,45 @@ static void sort_selected(struct playlist *playlist, gint(*compare) (const void
     PLAYLIST_HAS_CHANGED;
 }
 
-void playlist_sort_by_filename(gint playlist_num, gint(*compare) (const gchar * a, const gchar * b))
+void playlist_sort_by_filename (gint playlist_num, gint (* compare)
+ (const gchar * a, const gchar * b))
 {
     DECLARE_PLAYLIST;
 
     LOOKUP_PLAYLIST;
 
-    current_filename_compare = compare;
-    sort(playlist, filename_compare);
+    sort (playlist, filename_compare, compare);
 }
 
-void playlist_sort_by_tuple(gint playlist_num, gint(*compare) (const Tuple * a, const Tuple * b))
+void playlist_sort_by_tuple (gint playlist_num, gint (* compare)
+ (const Tuple * a, const Tuple * b))
 {
     DECLARE_PLAYLIST;
-    gint entries, count;
-
     LOOKUP_PLAYLIST;
-    entries = index_count(playlist->entries);
-
-    for (count = 0; count < entries; count++)
-    {
-        struct entry *entry = index_get(playlist->entries, count);
-
-        check_scanned (playlist, entry);
-    }
-
-    current_tuple_compare = compare;
-    sort(playlist, tuple_compare);
+    check_all_scanned (playlist);
+    sort (playlist, tuple_compare, compare);
 }
 
-void playlist_sort_selected_by_filename(gint playlist_num, gint(*compare) (const gchar * a, const gchar * b))
+void playlist_sort_selected_by_filename (gint playlist_num, gint (* compare)
+ (const gchar * a, const gchar * b))
 {
     DECLARE_PLAYLIST;
 
     LOOKUP_PLAYLIST;
 
-    current_filename_compare = compare;
-    sort_selected(playlist, filename_compare);
+    sort_selected (playlist, filename_compare, compare);
 }
 
-void playlist_sort_selected_by_tuple(gint playlist_num, gint(*compare) (const Tuple * a, const Tuple * b))
+void playlist_sort_selected_by_tuple (gint playlist_num, gint (* compare)
+ (const Tuple * a, const Tuple * b))
 {
     DECLARE_PLAYLIST;
-    gint entries, count;
-
     LOOKUP_PLAYLIST;
-    entries = index_count(playlist->entries);
-
-    for (count = 0; count < entries; count++)
-    {
-        struct entry *entry = index_get(playlist->entries, count);
-
-        if (entry->selected)
-            check_scanned (playlist, entry);
-    }
-
-    current_tuple_compare = compare;
-    sort_selected(playlist, tuple_compare);
+    check_selected_scanned (playlist);
+    sort_selected (playlist, tuple_compare, compare);
 }
 
-void playlist_reformat_titles()
+void playlist_reformat_titles (void)
 {
     gint playlist_num;
 
@@ -1459,26 +1542,60 @@ void playlist_rescan(gint playlist_num)
     METADATA_HAS_CHANGED;
 }
 
-gint64 playlist_get_total_length(gint playlist_num)
+void playlist_rescan_file (const gchar * filename)
+{
+    gint num_playlists = index_count (playlists);
+    gint playlist_num;
+
+    METADATA_WILL_CHANGE;
+
+    gchar * copy = NULL;
+    if (! uri_is_utf8 (filename, TRUE))
+        filename = copy = uri_to_utf8 (filename);
+
+    for (playlist_num = 0; playlist_num < num_playlists; playlist_num ++)
+    {
+        struct playlist * playlist = index_get (playlists, playlist_num);
+        gint num_entries = index_count (playlist->entries);
+        gint entry_num;
+
+        for (entry_num = 0; entry_num < num_entries; entry_num ++)
+        {
+            struct entry * entry = index_get (playlist->entries, entry_num);
+
+            if (! strcmp (entry->filename, filename))
+            {
+                entry_set_tuple (playlist, entry, NULL);
+                entry->failed = FALSE;
+            }
+        }
+    }
+
+    g_free (copy);
+
+    METADATA_HAS_CHANGED;
+}
+
+gint64 playlist_get_total_length (gint playlist_num, gboolean fast)
 {
     DECLARE_PLAYLIST;
-
     LOOKUP_PLAYLIST_RET (0);
 
+    if (! fast)
+        check_all_scanned (playlist);
+
     return playlist->total_length;
 }
 
-gint64 playlist_get_selected_length(gint playlist_num)
+gint64 playlist_get_selected_length (gint playlist_num, gboolean fast)
 {
     DECLARE_PLAYLIST;
-
     LOOKUP_PLAYLIST_RET (0);
 
-    return playlist->selected_length;
-}
+    if (! fast)
+        check_selected_scanned (playlist);
 
-void playlist_set_shuffle (gboolean on) /* currently not needed */
-{
+    return playlist->selected_length;
 }
 
 gint playlist_queue_count(gint playlist_num)
@@ -1601,6 +1718,23 @@ DONE:
     SELECTION_HAS_CHANGED;
 }
 
+void playlist_queue_delete_selected (gint playlist_num)
+{
+    DECLARE_PLAYLIST;
+    LOOKUP_PLAYLIST;
+
+    for (GList * node = playlist->queued; node != NULL; )
+    {
+        GList * next = node->next;
+        struct entry * entry = node->data;
+        if (entry->selected)
+            playlist->queued = g_list_delete_link (playlist->queued, node);
+        node = next;
+    }
+
+    SELECTION_HAS_CHANGED;
+}
+
 static gboolean shuffle_prev (struct playlist * playlist)
 {
     gint entries = index_count (playlist->entries), count;
@@ -1610,8 +1744,9 @@ static gboolean shuffle_prev (struct playlist * playlist)
     {
         struct entry * entry = index_get (playlist->entries, count);
 
-        if (entry->shuffle_num && entry->shuffle_num < playlist->shuffle_num &&
-         (found == NULL || entry->shuffle_num > found->shuffle_num))
+        if (entry->shuffle_num && (playlist->position == NULL ||
+         entry->shuffle_num < playlist->position->shuffle_num) && (found == NULL
+         || entry->shuffle_num > found->shuffle_num))
             found = entry;
     }
 
@@ -1619,7 +1754,6 @@ static gboolean shuffle_prev (struct playlist * playlist)
         return FALSE;
 
     playlist->position = found;
-    playlist->shuffle_num = found->shuffle_num;
     return TRUE;
 }
 
@@ -1629,14 +1763,7 @@ gboolean playlist_prev_song(gint playlist_num)
 
     LOOKUP_PLAYLIST_RET (FALSE);
 
-    if (playlist->queued != NULL)
-    {
-        if (playlist->position == playlist->queued->data)
-            return FALSE;
-
-        set_position (playlist, playlist->queued->data);
-    }
-    else if (cfg.shuffle)
+    if (cfg.shuffle)
     {
         if (! shuffle_prev (playlist))
             return FALSE;
@@ -1670,15 +1797,15 @@ static gboolean shuffle_next (struct playlist * playlist)
 
         if (! entry->shuffle_num)
             choice ++;
-        else if (entry->shuffle_num > playlist->shuffle_num && (found == NULL ||
-         entry->shuffle_num < found->shuffle_num))
+        else if (playlist->position != NULL && entry->shuffle_num >
+         playlist->position->shuffle_num && (found == NULL || entry->shuffle_num
+         < found->shuffle_num))
             found = entry;
     }
 
     if (found != NULL)
     {
         playlist->position = found;
-        playlist->shuffle_num = found->shuffle_num;
         return TRUE;
     }
 
@@ -1708,7 +1835,6 @@ static void shuffle_reset (struct playlist * playlist)
 {
     gint entries = index_count (playlist->entries), count;
 
-    playlist->shuffle_num = 0;
     playlist->last_shuffle_num = 0;
 
     for (count = 0; count < entries; count ++)
@@ -1730,14 +1856,14 @@ gboolean playlist_next_song(gint playlist_num, gboolean repeat)
     if (entries == 0)
         return FALSE;
 
-    if (playlist->position != NULL && playlist->position->queued)
+    /* If we have a song in queue, jump to it, _then_ remove it from queue */
+    if (playlist->queued != NULL)
     {
+        set_position (playlist, playlist->queued->data);
+
         playlist->queued = g_list_remove(playlist->queued, playlist->position);
         playlist->position->queued = FALSE;
     }
-
-    if (playlist->queued != NULL)
-        set_position (playlist, playlist->queued->data);
     else if (cfg.shuffle)
     {
         if (! shuffle_next (playlist))
@@ -1805,7 +1931,6 @@ void playlist_save_state (void)
 
         fprintf (handle, "playlist %d\n", playlist_num);
         fprintf (handle, "position %d\n", playlist_get_position (playlist_num));
-        fprintf (handle, "shuffled %d\n", playlist->shuffle_num);
         fprintf (handle, "last-shuffled %d\n", playlist->last_shuffle_num);
 
         for (count = 0; count < entries; count ++)
@@ -1855,7 +1980,7 @@ void playlist_load_state (void)
 {
     gchar scratch[512];
     FILE * handle;
-    gint playlist_num;
+    gint playlist_num, obsolete = 0;
 
     snprintf (scratch, sizeof scratch, "%s/" STATE_FILE,
      aud_paths[BMP_PATH_USER_DIR]);
@@ -1892,13 +2017,13 @@ void playlist_load_state (void)
         if (position >= 0 && position < entries)
             playlist->position = index_get (playlist->entries, position);
 
-        if (parse_integer ("shuffled", & playlist->shuffle_num))
+        if (parse_integer ("shuffled", & obsolete)) /* compatibility with 2.3 */
             parse_next (handle);
 
         if (parse_integer ("last-shuffled", & playlist->last_shuffle_num))
             parse_next (handle);
         else /* compatibility with 2.3 beta */
-            playlist->last_shuffle_num = playlist->shuffle_num;
+            playlist->last_shuffle_num = obsolete;
 
         for (count = 0; count < entries; count ++)
         {
